"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const functions_1 = tslib_1.__importDefault(require("./functions"));
const util_1 = require("../../util");
const contentfly_1 = require("../../util/cloud-function/contentfly");
jest.mock('../../util');
jest.mock('../../util/cloud-function/contentfly');
describe('Functions Command', () => {
    describe('init', () => {
        let originalParseMethod;
        let logMock;
        beforeEach(() => {
            originalParseMethod = functions_1.default.prototype['parse'];
            logMock = jest.fn();
            util_1.Logger.mockImplementation(() => ({
                log: logMock,
            }));
            functions_1.default.prototype['parse'] = jest
                .fn()
                .mockResolvedValueOnce({ flags: { 'data-dir': '/path/to/data/dir', port: '4000' } });
        });
        afterEach(() => {
            functions_1.default.prototype['parse'] = originalParseMethod;
            jest.resetAllMocks();
        });
        it('should initialize the command and set the sharedConfig values', async () => {
            const functionsCommand = new functions_1.default([], {});
            await functionsCommand.init();
            expect(functions_1.default.prototype['parse']).toHaveBeenCalledWith(functions_1.default);
            expect(functionsCommand['sharedConfig']).toEqual({
                projectBasePath: '/path/to/data/dir',
                port: 4000,
            });
        });
        it.each([{ portFlagInput: 'invalidPortInput' }, { portFlagInput: '999999' }, { portFlagInput: '-200' }])('should log an error and exit if the "port" flag input is invalid -> $portFlagInput', async ({ portFlagInput }) => {
            functions_1.default.prototype['parse'] = jest
                .fn()
                .mockResolvedValueOnce({ flags: { 'data-dir': '/path/to/data/dir', port: portFlagInput } });
            const exitMock = jest.spyOn(functions_1.default.prototype, 'exit').mockImplementation();
            const functionsCommand = new functions_1.default([], {});
            await functionsCommand.init();
            expect(functions_1.default.prototype['parse']).toHaveBeenCalledWith(functions_1.default);
            expect(logMock).toHaveBeenCalledWith('Invalid port number. Please provide a valid port number between 0 and 65535.', 'error');
            expect(exitMock).toHaveBeenCalledWith(1);
        });
        describe('init -> environment variable input handling', () => {
            const originalProcessEnv = Object.assign({}, process.env);
            afterEach(() => {
                process.env = originalProcessEnv;
            });
            it('should set the port based on the PORT environment variable', async () => {
                process.env.PORT = '5000';
                const functionsCommand = new functions_1.default([], {});
                await functionsCommand.init();
                expect(functions_1.default.prototype['parse']).toHaveBeenCalledWith(functions_1.default);
                expect(functionsCommand['sharedConfig']).toEqual({
                    projectBasePath: '/path/to/data/dir',
                    port: 5000,
                });
            });
            it.each([
                { portEnvironmentVariableValue: 'invalidPortInput' },
                { portEnvironmentVariableValue: '999999' },
                { portEnvironmentVariableValue: '-200' },
            ])('should log an error and exit if the "PORT" environment variable value is invalid -> $portEnvironmentVariableValue', async ({ portEnvironmentVariableValue }) => {
                process.env.PORT = portEnvironmentVariableValue;
                const exitMock = jest.spyOn(functions_1.default.prototype, 'exit').mockImplementation();
                const functionsCommand = new functions_1.default([], {});
                await functionsCommand.init();
                expect(functions_1.default.prototype['parse']).toHaveBeenCalledWith(functions_1.default);
                expect(logMock).toHaveBeenCalledWith('Invalid port number. Please provide a valid port number between 0 and 65535.', 'error');
                expect(exitMock).toHaveBeenCalledWith(1);
            });
        });
    });
    describe('run', () => {
        let serveCloudFunctionsMock;
        beforeEach(() => {
            serveCloudFunctionsMock = jest.fn();
            contentfly_1.Contentfly.mockImplementation(() => ({
                serveCloudFunctions: serveCloudFunctionsMock,
            }));
        });
        afterEach(() => {
            jest.resetAllMocks();
        });
        it('should call serveCloudFunctions with the correct port', async () => {
            const functionsCommand = new functions_1.default([], {});
            functionsCommand['sharedConfig'] = {
                projectBasePath: '/path/to/data/dir',
                port: 4000,
            };
            await functionsCommand.run();
            expect(contentfly_1.Contentfly).toHaveBeenCalledWith('/path/to/data/dir');
            expect(serveCloudFunctionsMock).toHaveBeenCalledWith(4000);
        });
    });
});
